#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2023 Brooks Su
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""To provide functions for reading keyboard in non-canonical mode.

There are 2 functions:
    getch(): Gets a character from stdin.
    getkey(): Calls getch() and transforms character to keycode.
Function keycodes in common using are defined in class Key.

A typical usage example for getkey():

    import termkey
    from termkey import Key

    key = termkey.getkey()
    if key == Key.SHIFT + Key.RIGHT:
        ...

Applicability: Implementations are base on XTerm specification and
termios, so they are just for POSIX terminal applications.
"""

import re
import sys
import termios
from enum import IntEnum


BLOCKING_ = -1

def getch(timeout: int = BLOCKING_) -> str:
    """Gets a character from stdin with non-canonical mode.

    Instead of waiting for a CR in canonical mode, function returns
    immediately once read a key. It also diabled echo and INTR signal
    when reading.

    Args:
        timeout: An integer to specify a waiting time in units of 1/10
            second. If timeout = BLOCKING_, function will block until
            read a charactor, otherwise return immediately if timeout
            occurs, regardless of whether a charactor is read or not.

    Returns:
        A character in string if successful reading. Otherwise returns
        an empty string '' when timeout.
    """

    in_fd = sys.stdin.fileno()
    # The Python termios flags is a list, as following:
    #   [iflag, oflag, cflag, lflag, ispeed, ospeed, cc]
    old_flags = termios.tcgetattr(in_fd)
    new_flags = old_flags[:]  # Makes a copy of flags
    new_flags[6] = old_flags[6][:]

    # Sets lflag(local flag): non-canonical mode, disable echo, and does
    # not generate INTR signal when Ctrl+C pressed.
    new_flags[3] &= ~(termios.ICANON | termios.ECHO | termios.ISIG)

    # Sets cc(control characters): At least reads 1 character in blocking
    # mode, otherwise 0 with a specific timeout.
    if timeout == BLOCKING_:
        new_flags[6][termios.VMIN] = 1
    else:
        new_flags[6][termios.VMIN] = 0
        # Sets minimal waiting time, in units of 1/10 second.
        new_flags[6][termios.VTIME] = timeout

    try:
        termios.tcsetattr(in_fd, termios.TCSADRAIN, new_flags)
        return sys.stdin.read(1)
    finally:
        termios.tcsetattr(in_fd, termios.TCSADRAIN, old_flags)


class Key(IntEnum):
    """Defines function key codes that returns by getkey().

    The key codes here are 16 bit integers: The single-byte keys are
    simply transformed by ord() function, and the multi-bytes CSI
    sequence keys are pesudo codes that begin from 0x101. Function key
    modifiers defined as bitmasks that could be combined with other key
    codes, like: CONTROL + SHIFT + F5

    The term 'CSI sequences key' here refers to key sequences lead with
    '\\033['(CSI) or '\\033O'(SS3) in XTerm specification.
    reference: https://www.xfree86.org/current/ctlseqs.html
    """

    NONE        = 0x0000
    BELL        = 0x07
    BACKSPACE   = 0x08
    TAB         = 0x09
    ENTER       = 0x0A
    ESC         = 0x1B
    SPACE       = 0x20
    DEL         = 0x7F
    CONTROL_A   = 0x01
    CONTROL_B   = 0x02
    CONTROL_C   = 0x03
    CONTROL_D   = 0x04
    CONTROL_E   = 0x05
    CONTROL_F   = 0x06
    CONTROL_G   = 0x07
    CONTROL_H   = 0x08
    CONTROL_I   = 0x09
    CONTROL_J   = 0x0A
    CONTROL_K   = 0x0B
    CONTROL_L   = 0x0C
    CONTROL_M   = 0x0D
    CONTROL_N   = 0x0E
    CONTROL_O   = 0x0F
    CONTROL_P   = 0x10
    CONTROL_Q   = 0x11
    CONTROL_R   = 0x12
    CONTROL_S   = 0x13
    CONTROL_T   = 0x14
    CONTROL_U   = 0x15
    CONTROL_V   = 0x16
    CONTROL_W   = 0x17
    CONTROL_X   = 0x18
    CONTROL_Y   = 0x19
    CONTROL_Z   = 0x1A

    # Pseudo codes for multi-bytes CSI sequence keys.
    UP          = 0x0101
    DOWN        = 0x0102
    RIGHT       = 0x0103
    LEFT        = 0x0104
    HOME        = 0x0105
    END         = 0x0106
    INSERT      = 0x0107
    DELETE      = 0x0108
    PAGEUP      = 0x0109
    PAGEDOWN    = 0x010A
    F1          = 0x010B
    F2          = 0x010C
    F3          = 0x010D
    F4          = 0x010E
    F5          = 0x010F
    F6          = 0x0110
    F7          = 0x0111
    F8          = 0x0112
    F9          = 0x0113
    F10         = 0x0114
    F11         = 0x0115
    F12         = 0x0116
    F13         = 0x0117
    F14         = 0x0118
    F15         = 0x0119
    F16         = 0x011A
    F17         = 0x011B
    F18         = 0x011C
    F19         = 0x011D
    F20         = 0x011E
    F0          = 0x011F

    # Pseudo codes for function key modifiers.
    SHIFT       = 0x0100
    ALT         = 0x0200
    CONTROL     = 0x0400
    META        = 0x0800
    OPTION      = 0x0800


_csi_sequences = {
    # XTerm sequences
    'A' : Key.UP,
    'B' : Key.DOWN,
    'C' : Key.RIGHT,
    'D' : Key.LEFT,
    'F' : Key.END,
    'H' : Key.HOME,
    'P' : Key.F1,
    'Q' : Key.F2,
    'R' : Key.F3,
    'S' : Key.F4,
    '1P' : Key.F1,
    '1Q' : Key.F2,
    '1R' : Key.F3,
    '1S' : Key.F4,

    # VT sequences
    '1~' : Key.HOME,
    '2~' : Key.INSERT,
    '3~' : Key.DELETE,
    '4~' : Key.END,
    '5~' : Key.PAGEUP,
    '6~' : Key.PAGEDOWN,
    '7~' : Key.HOME,
    '8~' : Key.END,
    '10~' : Key.F0,
    '11~' : Key.F1,
    '12~' : Key.F2,
    '13~' : Key.F3,
    '14~' : Key.F4,
    '15~' : Key.F5,
    '17~' : Key.F6,
    '18~' : Key.F7,
    '19~' : Key.F8,
    '20~' : Key.F9,
    '21~' : Key.F10,
    '23~' : Key.F11,
    '24~' : Key.F12,
    '25~' : Key.F13,
    '26~' : Key.F14,
    '28~' : Key.F15,
    '29~' : Key.F16,
    '31~' : Key.F17,
    '32~' : Key.F18,
    '33~' : Key.F19,
    '34~' : Key.F20,
}


# The XTerm encodes function key modifiers as parameters appended before the
# final character of the control sequence, and the modifiers are encoded in
# range of 2-16.
# The tuple '_modifiers' recorded those modifiers, and made their indices in
# the tuple equal to their codes.
# See also: http://www.leonerd.org.uk/hacks/fixterms/
_modifiers = ( 0, 0,
    Key.SHIFT,
    Key.ALT,
    Key.SHIFT + Key.ALT,
    Key.CONTROL,
    Key.SHIFT + Key.CONTROL,
    Key.ALT + Key.CONTROL,
    Key.SHIFT + Key.ALT + Key.CONTROL,
    Key.META,
    Key.META + Key.SHIFT,
    Key.META + Key.ALT,
    Key.META + Key.ALT + Key.SHIFT,
    Key.META + Key.CONTROL,
    Key.META + Key.CONTROL + Key.SHIFT,
    Key.META + Key.CONTROL + Key.ALT,
    Key.META + Key.CONTROL + Key.ALT + Key.SHIFT
)


def _match_csi_sequence(seq):
    seq = seq[1:]  # Skips the first character: '[' or 'O'.
    if seq in _csi_sequences:
        return _csi_sequences[seq]
    csi = re.match(r'(.+);(1[0-6]|[1-9])([@-~])$', seq)
    if csi is not None:
        reduct = csi.group(1) + csi.group(3)  # Omits argument of modifier.
        if reduct in _csi_sequences:
            return _modifiers[int(csi.group(2))] + _csi_sequences[reduct]
        # Omits all arguments and try again for convention reason.
        if csi.group(1) == '1' and csi.group(3) in _csi_sequences:
            return _modifiers[int(csi.group(2))] + _csi_sequences[csi.group(3)]
    return Key.NONE


_key_buffer = []

def getkey(timeout: int = BLOCKING_, raw: bool = False) -> Key | int:
    """Gets key(s) from getch() and tranforms it(them) from string to code.

    Function keys in form of CSI sequences will be tranformed to pseudo
    key codes that defined in class Key when 'raw' = False. All other
    unrecognized ESC sequence keys be treated as an ESC and leftover
    individual keys.

    Args:
        timeout: An argument passed to getch().
        raw: If True, getkey() will not try to tranform CSI sequence in
            keycode, but just return their code of ord() byte by byte.
            This is usually used for key testing purpose.

    Returns:
        Character's keycode from ord(), or function keycode that defined
        in class Key, or Key.NONE if timeout.
    """
    global _key_buffer

    key_ch = _key_buffer.pop(0) if _key_buffer else getch(timeout)
    if not key_ch:
        return Key.NONE
    key_code = ord(key_ch)
    if key_code == Key.ESC and not raw:
        seq = getch(0)
        if seq in '[O':
            # In CSI sequence for leading by '\033[' or '\033O'.
            key_ch = getch(0)
            while key_ch:
                seq += key_ch
                if 0x40 <= ord(key_ch) <= 0x7E:
                    # CSI terminal character are in range of 0x40-0x7E.
                    fcode = _match_csi_sequence(seq)
                    if fcode != Key.NONE:
                        return fcode
                    break
                key_ch = getch(0)
        # Puts the left keys to the buffer for unknown sequence.
        _key_buffer += list(seq)
    return key_code


def _test_termkey():
    """Continuously gets keys and prints their codes and characters to
    stdout until read a CONTROL-X.

    It is just for key sequences testing purpose. Sets command argument '-r'
    will make it only display raw key sequences.
    """
    raw = (len(sys.argv) > 1 and sys.argv[1] == '-r')
    print('Press any key to get code, CONTROL-X to exit.')
    key = getkey()
    while key != Key.CONTROL_X:
        print(f'code: {key:04x}, char: {chr(key)!r}')
        key = getkey(BLOCKING_, raw)
    input('Press enter to exit.')  # verifys the setting recovery.


if __name__ == '__main__':
    _test_termkey()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2023 Brooks Su
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Sets 256-color display attributes of character terminal.

There are three pairs of functions to set or restore colors according
to their literal name:

    set_text_color(color)
    reset_text_color()

    set_background_color(color)
    reset_background_color()

    set_color(text_color, background_color)
    reset_color()

Enum class Color defined constants of some basic colors and grayscales,
and more colors can be generated by function color256(r, g, b).

A simple sample as following:

    import color256
    from color256 import Color

    color256.set_color(Color.GOLD, Color.BLUE)
    print('Hello, color256!')
    color = color256.color256(4, 3, 1) # get a bronze color
    color256.set_text_color(color)
    ...
    color256.reset_color()

Reference: https://en.wikipedia.org/wiki/ANSI_escape_code
"""

import enum

_CSI_LEAD = '\033['

def color256(red: int, green: int, blue: int) -> int:
    """Returns a 256-color number according to the given RGB parameters.

    Args:
        All parameters' valid values must be in range of 0~5.

    Raises:
        ValueError: Argument value out of range.
    """
    if not (0 <= red <= 5 and 0 <= green <= 5 and 0 <= blue <= 5):
        raise ValueError('Argument value out of range.')
    return 16 + red * 36 + green * 6 + blue


class Color(enum.IntEnum):
    """Enumerates some common constants of colors.
    """
    BLACK = 0
    RED = 1
    GREEN = 2
    YELLOW = 3
    BLUE = 4
    MAGENTA = 5
    CYAN = 6
    WHITE = 7
    BRIGHT_BLACK = 8
    BRIGHT_RED = 9
    BRIGHT_GREEN = 10
    BRIGHT_YELLOW = 11
    BRIGHT_BLUE = 12
    BRIGHT_MAGENTA = 13
    BRIGHT_CYAN = 14
    BRIGHT_WHITE = 15

    GRAYSCALE_DARKEST = 232
    GRAYSCALE_DARK = 238
    GRAYSCALE_MIDDLE = 243
    GRAYSCALE_LIGHT = 249
    GRAYSCALE_LIGHTEST = 255
    GRAY = color256(3, 3, 3)
    SILVER = color256(4, 4, 4)

    TAN = color256(5, 4, 3)
    BRONZE = color256(4, 3, 1)
    COPPER = BRONZE
    COFFEE = color256(2, 1, 0)
    CHOCOLATE = color256(4, 2, 1)
    BROWN = color256(3, 1, 1)

    PINK = color256(5, 3, 3)
    HOT_PINK = color256(5, 2, 4)
    DEEP_PINK = color256(5, 0, 3)
    TOMATO = color256(5, 2, 2)
    INDIAN_RED = TOMATO
    MAROON = RED
    CRIMSON = color256(5, 0, 1)
    SCARLET = CRIMSON

    VIOLET = color256(3, 0, 5)
    BRIGHT_VIOLET = color256(5, 3, 5)
    ORCHID = color256(3, 1, 5)
    BRIGHT_ORCHID = color256(4, 2, 4)
    PLUM = color256(5, 4, 5)
    LAVENDER = color256(4, 4, 5)
    MEDIUM_PURPLE = color256(3, 2, 5)
    PURPLE = color256(3, 0, 3)

    INDIGO = color256(1, 0, 4)
    NAVY = color256(0, 0, 3)
    ROYAL_BLUE = color256(2, 3, 5)
    SKY_BLUE = color256(3, 5, 5)
    DEEP_SKY_BLUE = color256(0, 4, 5)
    AZURE = color256(4, 5, 5)

    TURQUOISE = color256(2, 5, 4)
    SPRING_GREEN = color256(2, 4, 3)
    SEA_GREEN = color256(1, 4, 4)
    DEEP_SEA_GREEN = color256(1, 3, 2)
    OLIVE = color256(3, 3, 0)

    BEIGE = color256(5, 5, 4)
    IVORY = BEIGE
    KHAKI = color256(5, 5, 3)
    DEEP_KHAKI = color256(4, 4, 2)
    GOLD = color256(5, 4, 0)
    ORANGE = color256(5, 3, 0)
    DEEP_ORANGE = color256(5, 2, 0)


def set_text_color(color: int):
    """Sets text(foreground) color of terminal.
    """
    print(f'{_CSI_LEAD}38;5;{color}m', end='')


def reset_text_color():
    """Restores text color to the default value of terminal.
    """
    print(f'{_CSI_LEAD}39m', end='')


def set_background_color(color: int):
    """Sets background color of terminal.
    """
    print(f'{_CSI_LEAD}48;5;{color}m', end='')


def reset_background_color():
    """Restores background color to the default value of terminal.
    """
    print(f'{_CSI_LEAD}49m', end='')


def set_color(text_color: int, background_color: int):
    """Sets text(foreground) color to 'text_color' and background color
    to 'background_color'.
    """
    print(f'{_CSI_LEAD}38;5;{text_color}m{_CSI_LEAD}48;5;{background_color}m',
          end='')


def reset_color():
    """Restores text and background color to the default value of terminal.
    """
    print(f'{_CSI_LEAD}39;49m', end='')


def _test_color256():
    def color_block(color, width):
        set_background_color(color)
        print(' ' * width, end='')
        reset_background_color()

    def text_color():
        for i in range(6):
            for char in 'Hello, color256!':
                set_text_color(ord(char) % 32 + 20 + i * 36)
                print(char, end='')
            print('')
        reset_text_color()

    def grayscale():
        for color in range(Color.GRAYSCALE_DARKEST,
                           Color.GRAYSCALE_LIGHTEST + 1):
            color_block(color, 3)
        print('')

    def rgb():
        for red in range(6):
            for green in range(6):
                for blue in range(6):
                    color_block(color256(red, green, blue), 2)
            print('')

    def constant_color():
        for color in sorted(Color):
            print(f'{color.name:18s}: {color.value:3d} ', end='')
            color_block(color, 12)
            print('')

    text_color()
    grayscale()
    rgb()
    constant_color()


if __name__ == '__main__':
    _test_color256()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#   Copyright 2023 Brooks Su
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Wrapper functions of CSI(Control Sequence Introducer) sequences about
cursor and screen.

The sequences of following functions based on are not part of the ANSI
specification, but be supported by most of terminals.

    show_cursor()
    hide_cursor()
    switch_screen()
    restore_screen()

A several of additional functions are provided for text composing:

    v_composing()
    downward_seq()
    vert_seq()
    rect_border_seq()

Reference: https://en.wikipedia.org/wiki/ANSI_escape_code
           https://www.xfree86.org/current/ctlseqs.html
"""

from wcwidth import wcswidth


_CSI_LEAD = '\033['


def cursor_up_seq(num: int = 1):
    """Returns a sequence string that can move the cursor up 'num' rows.
    """
    return f'{_CSI_LEAD}{num}A'


def cursor_up(num: int = 1):
    """Moves the cursor up 'num' rows.
    """
    print(cursor_up_seq(num), end='', flush=True)


def cursor_down_seq(num: int = 1):
    """Returns a sequence string that can move the cursor down 'num' rows.
    """
    return f'{_CSI_LEAD}{num}B'


def cursor_down(num: int = 1):
    """Moves the cursor down 'num' rows.
    """
    print(cursor_down_seq(num), end='', flush=True)


def cursor_right_seq(num: int = 1):
    """Returns a sequence string that can move the cursor 'num' columns
    to right.
    """
    return f'{_CSI_LEAD}{num}C'


def cursor_right(num: int = 1):
    """Moves the cursor 'num' columns to right.
    """
    print(cursor_right_seq(num), end='', flush=True)


def cursor_left_seq(num: int = 1):
    """Returns a sequence string that can move the cursor 'num' columns
    to left.
    """
    return f'{_CSI_LEAD}{num}D'


def cursor_left(num: int = 1):
    """Moves the cursor 'num' columns to left.
    """
    print(cursor_left_seq(num), end='', flush=True)


def cursor_pos_seq(row: int, col: int):
    """Returns a sequence string that can move the cursor to the coordinate
    (row, col).
    """
    return f'{_CSI_LEAD}{row};{col}H'


def cursor_pos(row: int, col: int):
    """Moves the cursor to the coordinate (row, col).
    """
    print(cursor_pos_seq(row, col), end='', flush=True)


def putmsg(row: int, col: int, message: str, *args, **kwargs):
    """Moves the cursor to the coordinate (row, col), and prints message.
    """
    print(f'{cursor_pos_seq(row, col)}{message}',
          *args, **kwargs, end='', flush=True)


def clear_screen():
    """Clears current screen buffer.
    """
    print(f'{_CSI_LEAD}2J', end='', flush=True)


def scroll_up(num: int = 1):
    """Scroll whole page up by 'num' rows. New rows are added at bottom.
    """
    print(f'{_CSI_LEAD}{num}S', end='', flush=True)


def scroll_down(num: int = 1):
    """Scroll whole page down by 'num' rows. New rows are added at top.
    """
    print(f'{_CSI_LEAD}{num}T', end='', flush=True)


def show_cursor():
    """Shows the cursor on screen.
    """
    print(f'{_CSI_LEAD}?25h', end='', flush=True)


def hide_cursor():
    """Hides the cursor from screen.
    """
    print(f'{_CSI_LEAD}?25l', end='', flush=True)


def switch_screen():
    """Uses alternate screen buffer to display message, it makes the content
    in the normal screen buffer remaining unchange.
    """
    print(f'{_CSI_LEAD}?47h', end='', flush=True)


def restore_screen():
    """Switches back to the normal screen buffer.
    """
    print(f'{_CSI_LEAD}?47l', end='', flush=True)


_comp_funcs = {
    'h': cursor_left_seq,
    'l': cursor_right_seq,
    'k': cursor_up_seq,
    'j': cursor_down_seq,
}

def v_composing(comp_seq: str) -> str:
    r"""Generates text composing sequence.

    The parameter 'comp_seq' can use a series of VI like commands(hlkj)
    to specify relative cursor movements, using ':' command to insert
    text, and an ESC(\x1b) to back command mode. A numberic prefix can be
    added to the command to specify repetition times.

    Following sample code displays three big colorful greetings:

        from cursor import putmsg, v_composing

        _P1 = ':{0}\x1b3hj'
        _P15 = ':{0}\x1b6l:{0}\x1b11hj'
        _DASH = '5:{0}\x1b11hj'
        _NEXT_POS = '5k17l'

        LETTERS = {
            'E': v_composing(f'{(_DASH + _P1) * 2}{_DASH}{_NEXT_POS}'),
            'H': v_composing(f'{_P15 * 2}{_DASH}{_P15 * 2}{_NEXT_POS}'),
            'L': v_composing(f'{_P1 * 4}{_DASH}{_NEXT_POS}'),
            'O': v_composing(f'{_DASH}{_P15 * 3}{_DASH}{_NEXT_POS}'),
        }

        greeting = ''.join(map(LETTERS.get, 'HELLO'))
        putmsg(3, 20, greeting.format('\u2b50'))
        putmsg(9, 14, greeting.format('\u2b55'))
        putmsg(15, 8, greeting.format('\U0001f7e2'))

    Returns:
        A text composing sequnce that can be output to the screen in the
        expected shape.

    Raises:
        ValueError: Invalid composing sequence.
    """
    global _comp_funcs
    result = []
    index = repeat = 0

    while index < len(comp_seq):
        cmd = comp_seq[index]
        index += 1
        if cmd.isdigit():
            repeat = repeat * 10 + int(cmd)
        elif cmd in _comp_funcs:
            result.append(_comp_funcs[cmd](repeat if repeat else 1))
            repeat = 0
        elif cmd == ':':  # In insertion mode
            esc = comp_seq.find('\x1b', index)
            if esc < 0:
                result.append(comp_seq[index:] * (repeat if repeat else 1))
                break
            result.append(comp_seq[index:esc] * (repeat if repeat else 1))
            repeat = 0
            index = esc + 1
        elif cmd != '\x1b':
            raise ValueError('Invalid composing sequence')
        # else: ignores redundant ESC(\x1b)
    return ''.join(result)


def downward_seq(text: str, back_cols: int) -> str:
    r"""Generates a text sequence that makes subsequent text to be output
    in downward direction.

    After the result text be output to screen, the cursor takes 'back_cols'
    columns back from the end position of the output and moves to the next
    line.

    The following example code draws a triangle on screen:

        putmsg(1, 40, downward_seq('\u2b50', 3) * 24)
        putmsg(1, 40, downward_seq('\u2b50', 1) * 24)
        putmsg(24, 17, '\u2b50' * 24)
    """
    return v_composing(f':{text}\x1b{back_cols}hj')


def vert_seq(text:str) -> str:
    """Generates a text sequence that makes subsequent text be output in
    vertical downward direction.

    It is just a simple wapper of the downward_seq().
    """
    return downward_seq(text, wcswidth(text))


def rect_border_seq(width: int, height: int, sym: str) -> str:
    """Generates a text sequence that draws a rectangle border.
    """
    sym_w = wcswidth(sym)
    hori_border = f':{sym * width}\x1b{sym_w * width}hj'
    vert_borders = ((f':{sym}\x1b{sym_w * (width - 2)}l'
                     f':{sym}\x1b{sym_w * width}hj') * (height - 2))
    return v_composing(hori_border + vert_borders + hori_border)


def _test_cursor():
    def greeting_show():
        p10 = ':{0}\x1b3hj'
        p00001 = '8l:{0}\x1b11hj'
        p10001 = ':{0}\x1b6l:{0}\x1b11hj'
        p1001 = ':{0}\x1b4l:{0}\x1b9hj'
        dash = '5:{0}\x1b11hj'
        next_pos = '5k17l'
        letters = {
            'C': v_composing(f'{dash}{p10 * 3}{dash}{next_pos}'),
            'E': v_composing(f'{(dash + p10) * 2}{dash}{next_pos}'),
            'H': v_composing(f'{p10001 * 2}{dash}{p10001 * 2}{next_pos}'),
            'L': v_composing(f'{p10 * 4}{dash}{next_pos}'),
            'O': v_composing(f'{dash}{p10001 * 3}{dash}{next_pos}'),
            'R': v_composing(f'{dash}{p10001}{dash}{p1001}{p10001}{next_pos}'),
            'S': v_composing(f'{dash}{p10}{dash}{p00001}{dash}{next_pos}'),
            'U': v_composing(f'{p10001 * 4}{dash}{next_pos}'),
        }
        hello = ''.join(map(letters.get, 'HELLO'))
        cursor = ''.join(map(letters.get, 'CURSOR'))
        putmsg(4, 20, hello.format('\U0001f7e4'))
        putmsg(10, 14, hello.format('\U0001f7e0'))
        putmsg(16, 8, cursor.format('\U0001f7e1'))

    def frame_show():
        putmsg(4, 2, rect_border_seq(39, 18, '\u2b50'))
        putmsg(1, 40, downward_seq('\U0001f339', 3) * 24)
        putmsg(1, 40, downward_seq('\U0001f339', 1) * 24)
        putmsg(24, 17, '\U0001f339' * 24)

    switch_screen()
    clear_screen()
    hide_cursor()
    try:
        greeting_show()
        cursor_pos(24, 1)
        input('Enter to continue.')
        clear_screen()
        frame_show()
        cursor_pos(24, 1)
        input('Enter to exit.')
    finally:
        show_cursor()
        restore_screen()


if __name__ == '__main__':
    _test_cursor()



# -*- coding: utf-8 -*-

#   Copyright 2023 Brooks Su
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""Collects some common icons in unicode character set.
"""

from enum import StrEnum


class UnicodeIcon(StrEnum):
    """Collects some common icons in unicode character set.
    """
    CAFE = '\u2615'
    WHITE_CIRCLE = '\u26aa'
    BLACK_CIRCLE = '\u26ab'
    FOOTBALL = '\u26bd'
    STOP = '\u26d4'
    GREEN_TICK = '\u2705'
    SMALL_WHITE_STAR = '\u2729'
    SMALL_PINWHEEL_STAR = '\u272f'
    SNOW_FLAKE = '\u2744'
    RED_CROSS = '\u274c'
    GREEN_CROSS = '\u274e'
    BLACK_SQUARE = '\u2b1b'
    WHITE_SQUARE = '\u2b1c'
    GOLD_STAR = '\u2b50'
    RED_RING = '\u2b55'

    SOS = '\U0001f198'
    RAINBOW = '\U0001f308'
    DARK_FACE = '\U0001f31a'
    GOLD_FACE = '\U0001f31d'
    TULIP = '\U0001f337'
    ROSE = '\U0001f339'
    FLOWER = '\U0001f33c'
    CORN = '\U0001f33d'
    MUSHROOM = '\U0001f344'
    TOMATO = '\U0001f345'
    RED_APPLE = '\U0001f34e'
    GREEN_APPLE = '\U0001f34f'
    PEAR = '\U0001f350'
    PEACH = '\U0001f351'
    CHERRY = '\U0001f352'
    HAM = '\U0001f356'
    GIFT = '\U0001f381'
    PUMPKIN = '\U0001f383'
    SAI = '\U0001f3ad'
    DICE = '\U0001f3b2'
    BASKETBALL = '\U0001f3c0'
    VOLLEYBALL = '\U0001f3d0'
    MICE = '\U0001f401'
    OCTOPUS = '\U0001f419'
    TIGER = '\U0001f42f'
    FROG = '\U0001f438'
    BEAR = '\U0001f43b'
    PANDA = '\U0001f43c'
    CROWN = '\U0001f451'
    JELLYFISH = '\U0001f47b'
    ALIEN = '\U0001f47d'
    SKULL = '\U0001f480'
    KEY = '\U0001f511'
    LOCK = '\U0001f512'
    RED_CIRCLE = '\U0001f534'
    BLUE_CIRCLE = '\U0001f535'
    ORANGE_DIAMOND = '\U0001f536'
    BLUE_DIAMOND = '\U0001f537'
    SMALL_ORANGE_DIAMOND = '\U0001f538'
    SMALL_BLUE_DIAMOND = '\U0001f539'
    ORANGE_CIRCLE = '\U0001f7e0'
    YELLOW_CIRCLE = '\U0001f7e1'
    GREEN_CIRCLE = '\U0001f7e2'
    MAGENTA_CIRCLE = '\U0001f7e3'
    BROWN_CIRCLE = '\U0001f7e4'
    RED_SQUARE = '\U0001f7e5'
    BLUE_SQUARE = '\U0001f7e6'
    ORANGE_SQUARE = '\U0001f7e7'
    YELLOW_SQUARE = '\U0001f7e8'
    GREEN_SQUARE = '\U0001f7e9'
    MAGENTA_SQUARE = '\U0001f7ea'
    BROWN_SQUARE = '\U0001f7eb'
    CRAB = '\U0001f980'
    BUTTERFLY = '\U0001f98b'
    SWAN = '\U0001f9a2'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from glob import glob
from os.path import basename, splitext
from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as f:
    readme_md = f.read()

setup(
    name='ltermio',
    version='0.1.5',
    description='A Lightweight POSIX terminal I/O library',
    long_description=readme_md,
    long_description_content_type='text/markdown',
    python_requires='>=3.6',
    license='Apache Software License',

    author='brookssu',
    author_email='yipeng00@gmail.com',
    url='https://github.com/brookssu/ltermio',

    packages=find_packages(where='src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'cursor-demo = cursor:_test_cursor',
            'color256-demo = color256:_test_color256',
            'termkey-demo = termkey:_test_termkey',
        ]
    },
    keywords=[
        'ltermio',
        'cursor',
        'termkey',
        'color256',
        'termios',
        'curses',
        'terminal',
        'color',
        'colour',
        'keyboard',
        'ansi',
        'CSI',
        'text composing',
        'Esc sequence',
        'POSIX terminal',
    ],
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 3.6',
        'Environment :: Console',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
